package api_we

import (
	"dc_golang_server_1/api"
	"github.com/gin-gonic/gin"
	"net/http"
	"time"
)

type bindCardUserRes struct {
	Master  bool    `json:"master" example:"false"` //true:本来就是主家长，false:已成功绑定成副家长
	ID      uint32  `json:"id"`                     // 学生ID
	Name    string  `json:"name"`
	Sex     uint8   `json:"sex"`                          // 0女 1男
	Area    string  `json:"area" example:"四川省成都市温江区实验中学"` // 区域名字
	Grade   int8    `json:"grade"`                        // "1~9对应一年级~九年级，10~12对应高一~高三"
	Class   int8    `json:"class"`
	StuNum  string  `json:"stuNum"`
	Service []uint8 `json:"service" example:"1,2,2,0,0,0"` // 对应6个数字对应公话、定位、饮水、卡消费、洗浴、洗衣机图标(0不显示,1正常,2显示但提示校区暂未开通该服务)
}

// BindCardUser 通过邀请码绑定学生
// @Tags 家长微信
// @Summary  通过邀请码绑定学生
// @Description 传学生的邀请码,后台返回学生信息
// @Produce json
// @Param share_code path string true "学生的邀请码"
// @Success 0 {object} bindCardUserRes "学生信息"
// @Failure 30 "邀请码无效"
// @Failure 31 "邀请码已过期"
// @Router /we/bindcarduser/{share_code} [POST]
// @Security ApiKeyAuth
func BindCardUser(c *gin.Context) {
	res := bindCardUserRes{}
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": res,
	})
}

// DeleteBindCardUser 副家长解绑学生
// @Tags 家长微信
// @Summary  副家长解绑学生
// @Description 传学生ID至后台,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Success 0
// @Router /we/bindcarduser/{id} [DELETE]
// @Security ApiKeyAuth
func DeleteBindCardUser(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

type wechatCardUsersTrendType struct {
	ServiceType uint8  `json:"service"` //服务项目 1公话2定位3饮水4卡消费5洗浴6洗衣机 决定UI图标及颜色等
	TrendName   string `json:"title"`   //显示在第一行
	Content     string `json:"content"` //显示在第二行
	Time        string `json:"time"`    //显示在第三行
}

// GetCardUsersTrend 微信学生首页,获取近10条用户最新动态
// @Tags 家长微信
// @Summary  微信学生首页,获取近20条用户最新动态
// @Description 微信学生首页,获取近20条用户最新动态
// @Param id path integer true "学生ID"
// @Product json
// @Success 200 {object} wechatCardUsersTrendType "学生动态信息"
// @Router /we/trend/{id} [GET]
// @Security ApiKeyAuth
func GetCardUsersTrend(c *gin.Context) {
	//id, _ := strconv.Atoi(c.Param("id"))
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": [20]wechatCardUsersTrendType{},
	})
}

type slaveWeUserRes struct {
	// 微信UnionID
	WeUserID uint32 `json:"weUserID"`
	// 微信昵称
	Nickname string `json:"nickname"`
	// 头像URL
	AvatarURL string `json:"avatarURL"`
	// 手机号
	Phone string `json:"phone"`
	// 绑定时间
	CreateAt time.Time `json:"createAt"`
}

//type getBindCardUserRes struct {
//	// 副家长微信信息列表
//	WeUser []slaveWeUserRes `json:"weUser"`
//	// 学生的邀请码,长度固定16,若为空,则表示邀请码开关关闭
//	ShareCode string `json:"shareCode"`
//}

// GetBindCardUser 获取学生副家长列表(仅主家长有权限)
// @Tags 家长微信
// @Summary  获取学生副家长列表(仅主家长有权限)
// @Description 传学生ID至后台,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Success 0 {object} []slaveWeUserRes "副家长信息列表"
// @Router /we/bindcarduser/{id} [GET]
// @Security ApiKeyAuth
func GetBindCardUser(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": []slaveWeUserRes{},
	})
}

// CardUserShareCode 获取邀请码(仅主家长有权限)
// @Tags 家长微信
// @Summary  获取邀请码(仅主家长有权限)
// @Description 主家长生成邀请码，有效时长可选1分钟~60分钟
// @Produce json
// @Param id path integer true "学生ID"
// @Param time query integer false "有效时长,1~60分钟,不填则为60分钟" default(60)
// @Success 0 {object} string "邀请码内容"
// @Router /we/sharecode/{id} [GET]
// @Security ApiKeyAuth
func CardUserShareCode(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": "todo",
	})
}

//// PutCardUserShareOn 打开邀请码开关(仅主家长有权限)
//// @Tags 家长微信
//// @Summary  打开邀请码开关(仅主家长有权限)
//// @Description 传学生ID至后台,后台返回成功失败
//// @Produce json
//// @Param id path integer true "学生ID"
//// @Success 0
//// @Router /we/cardusershareon/{id} [PUT]
//// @Security ApiKeyAuth
//func PutCardUserShareOn(c *gin.Context) {
//	c.JSON(http.StatusOK, gin.H{
//		"code": api.Success,
//	})
//}

//// PutCardUserShareOff 关闭邀请码开关(仅主家长有权限)
//// @Tags 家长微信
//// @Summary 关闭邀请码开关(仅主家长有权限)
//// @Description 传学生ID至后台,后台返回成功失败
//// @Produce json
//// @Param id path integer true "学生ID"
//// @Success 0
//// @Router /we/cardusershareoff/{id} [PUT]
//// @Security ApiKeyAuth
//func PutCardUserShareOff(c *gin.Context) {
//	c.JSON(http.StatusOK, gin.H{
//		"code": api.Success,
//	})
//}

// DeleteSlaveWeUser 删除副家长(仅主家长有权限)
// @Tags 家长微信
// @Summary 删除副家长(仅主家长有权限)
// @Description 传学生ID和微信UnionID,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Param weUserID query integer true "副家长微信weUserID"
// @Success 0
// @Router /we/slaveweuser/{id} [DELETE]
// @Security ApiKeyAuth
func DeleteSlaveWeUser(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

//// DeleteAllSlaveWeUser 清空所有副家长(仅主家长有权限)
//// @Tags 家长微信
//// @Summary 删除副家长(仅主家长有权限)
//// @Description 传学生ID和微信UnionID,后台返回成功失败
//// @Produce json
//// @Param id path integer true "学生ID"
//// @Param unionid query string true "副家长微信UnionID"
//// @Success 0
//// @Router /we/allslaveweuser/{id} [DELETE]
//// @Security ApiKeyAuth
//func DeleteAllSlaveWeUser(c *gin.Context) {
//	c.JSON(http.StatusOK, gin.H{
//		"code": api.Success,
//	})
//}

type PutCardUserReq struct {
	// 必填,学生名字,maxLen=16
	Name string `json:"name" example:"汤圆"`
	// 必填，"1~9对应一年级~九年级，10~12对应高一~高三"
	Grade int8 `json:"grade"`
	// 选填,0女1男
	Sex uint8 `json:"sex,omitempty"`
	// 选填,班级,number
	Class int8 `json:"class,omitempty"`
	// 选填,学号,字符串,maxLen=32
	StudentNo string `json:"stuNum,omitempty"`
}

// PutCardUser 修改学生信息
// @Tags 家长微信
// @Summary  修改学生信息
// @Description 传学生id及新信息至后台,后台返回成功失败
// @Accept  json
// @Produce json
// @Param id path integer true "学生ID"
// @Param data body PutCardUserReq true "学生信息"
// @Success 0
// @Router /we/carduser/{id} [PUT]
// @Security ApiKeyAuth
func PutCardUser(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

// CardLoss 卡挂失/解挂
// @Tags 家长微信
// @Summary  卡挂失/解挂
// @Description 传学生id及卡类型至后台,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Param loss query bool true "true挂失,false解挂"
// @Param type query integer true "卡类型:1电话卡,2定位卡,3IC卡(饮水、消费、洗浴、洗衣机均为IC卡,固任何一个服务的卡挂失状态改变,其余也跟随改变)"
// @Success 0
// @Router /we/cardloss/{id} [PUT]
// @Security ApiKeyAuth
func CardLoss(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

// CardBind 绑定卡
// @Tags 家长微信
// @Summary  绑定卡
// @Description 传学生id及卡类型、卡号至后台,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Param code query string true "卡号"
// @Param type query integer true "卡类型:1电话卡,2定位卡,3IC卡(饮水、消费、洗浴、洗衣机均为IC卡,固任何一个服务的卡挂失状态改变,其余也跟随改变)"
// @Success 0
// @Router /we/cardbind/{id} [PUT]
// @Security ApiKeyAuth
func CardBind(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

// DeleteCardBind 解绑卡
// @Tags 家长微信
// @Summary  解绑卡
// @Description 传学生id及卡类型至后台,后台返回成功失败
// @Produce json
// @Param id path integer true "学生ID"
// @Param type query integer true "卡类型:1电话卡,2定位卡,3IC卡(饮水、消费、洗浴、洗衣机均为IC卡,固任何一个服务的卡挂失状态改变,其余也跟随改变)"
// @Success 0
// @Router /we/cardbind/{id} [DELETE]
// @Security ApiKeyAuth
func DeleteCardBind(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

type familyNumStruct struct {
	Phone string `json:"phone"`
	Name  string `json:"name"`
}

type getPhoneCardStatusRes struct {
	Status     uint8             `json:"status"`     // 状态 0未绑定卡   1 正常使用   2 服务已到期 3 已挂失
	SimID      string            `json:"simID"`      // 卡号
	ExpiryAt   time.Time         `json:"expiryAt"`   // 到期时间
	MonthMin   uint16            `json:"monthMin"`   // 月套餐使用分钟数
	MonthPack  uint16            `json:"monthPack"`  // 月套餐分钟数
	Changeable uint8             `json:"changeable"` // 本月可修改亲情号次数
	Family     []familyNumStruct `json:"family"`     // 亲情号列表
}

// GetPhoneCardStatus 获取学生公话卡状态
// @Tags 家长微信
// @Summary  获取学生公话卡状态
// @Description 传学生ID至后台,后台返回卡信息
// @Produce json
// @Param id path integer true "学生ID"
// @Success 0 {object} getPhoneCardStatusRes "公话卡状态信息"
// @Router /we/phonecardstatus/{id} [GET]
// @Security ApiKeyAuth
func GetPhoneCardStatus(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": getPhoneCardStatusRes{},
	})
}

type getCallRecordRes struct {
	// 被叫号码
	CalledNumber string `json:"number"`
	// 昵称
	CalledNickname string `json:"nickname"`
	// 通话开始时间
	CallStartAt time.Time `json:"startAt"`
	// 通话时长,单位(分钟)
	TalkTime uint8 `json:"time"`
}

// GetCallRecord 查询通话记录
// @Tags 家长微信
// @Summary  查询通话记录
// @Description 传学生ID至后台,后台返回通话记录列表
// @Produce json
// @Param id path integer true "学生ID"
// @Param pnum query integer true "页数"
// @Param psize query integer true "页条数"
// @Param start query string false "起始时间 2020-10-10 10:00:00"
// @Param end query string false "结束时间 2030-10-10 10:00:00"
// @Param asc query bool false "是否升序" default(false)
// @Success 0 {object} []getCallRecordRes "通话记录列表"
// @Router /we/callrecord/{id} [GET]
// @Security ApiKeyAuth
func GetCallRecord(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": [2]getCallRecordRes{},
	})
}

// AddFamilyNum 添加亲情号
// @Tags 家长微信
// @Summary  添加亲情号
// @Description 家长绑定亲情号接口
// @Accept  json
// @Produce json
// @Param id path integer true "学生ID"
// @Param data body familyNumStruct true "亲情号内容"
// @Success 0
// @Router /we/familynum [POST]
// @Security ApiKeyAuth
func AddFamilyNum(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

// PutFamilyNum 修改亲情号
// @Tags 家长微信
// @Summary  修改亲情号
// @Description 修改亲情号接口
// @Accept  json
// @Produce json
// @Param id path integer true "学生ID"
// @Param phone query string true "电话号码"
// @Param data body familyNumStruct true "亲情号内容"
// @Success 0
// @Router /we/familynum [PUT]
// @Security ApiKeyAuth
func PutFamilyNum(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}

// DeleteFamilyNum 删除亲情号
// @Tags 家长微信
// @Summary  删除亲情号
// @Description 删除亲情号接口
// @Accept  json
// @Produce json
// @Param id path integer true "学生ID"
// @Param phone query string true "电话号码"
// @Success 0
// @Router /we/familynum [DELETE]
// @Security ApiKeyAuth
func DeleteFamilyNum(c *gin.Context) {
	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
	})
}
