package api

import (
	"dc_golang_server_1/data_db_cache/cache"
	"dc_golang_server_1/util"
	"github.com/gin-gonic/gin"
	"math/rand"
	"net/http"
	"time"
)

const jeffWTKey = "jeffWT@666%b;'~+"

//const jeffWTTable = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"

const tokenClearLen = 20
const tokenLen = 28

type TokenResult struct {
	CreatTime   int64  //token创建时间
	UserID      uint32 //
	ExpireTime  uint32
	UserType    uint8 // 1微信家长  2后台用户  其余预留
	CheckResult uint8
}

// CreateToken 生成token,duration单位秒,=0表示不限时长
func CreateToken(creatTime int64, userID uint32, userType uint8, duration uint32) string {
	var expireTime uint32
	if duration == 0 {
		expireTime = 0xffffffff
	} else {
		expireTime = uint32(creatTime/1000000) + duration
	}
	var clear = make([]byte, tokenClearLen) //7字节创建时间 4字节结束时间 4字节有效时长 2字节CRC 3字节随机数预留
	r := rand.New(rand.NewSource(time.Now().UnixNano()))
	random := byte(r.Intn(256))

	clear[0] = ^byte(userID>>24) - random
	clear[1] = byte(expireTime>>16) - random
	clear[2] = byte(userID>>16) + random + 166
	clear[3] = ^byte(creatTime>>24) - random
	clear[4] = byte(creatTime>>48) - random + 23
	clear[5] = ^byte(expireTime >> 8)
	clear[6] = byte(creatTime>>32) + random + 66
	clear[7] = byte(expireTime>>24) + random + 99
	clear[8] = byte(creatTime>>16) + random - 88
	clear[9] = ^byte(userID)
	clear[10] = byte(creatTime) - 69
	clear[11] = byte(expireTime) + random + 27
	clear[12] = byte(userID>>8) - random - 33
	clear[13] = ^byte(creatTime>>40) - random - 28
	clear[14] = byte(creatTime>>8) - random - 69
	clear[15] = ^random + 0x5a
	clear[16] = userType - random
	clear[17] = 0x1a + random
	util.GetCRC(clear, 18)
	clear[18] = ^(clear[18] + random - 15)
	clear[19] = ^clear[19] - random

	return util.EncryptBytesToBase64URL(clear, jeffWTKey)
}

func DecryptToken(token string) (clearStruct TokenResult) {
	if len(token) != tokenLen {
		// todo logger token长度错误
		clearStruct.CheckResult = ErrorTokenExist
		return
	}
	clear, err := util.DecryptBase64URLToBytes(token, jeffWTKey)
	if err != nil {
		// todo logger 有人在非法请求
		clearStruct.CheckResult = ErrorTokenWrong
		return
	}
	if len(clear) != tokenClearLen {
		// todo logger 有人在非法请求
		clearStruct.CheckResult = ErrorTokenWrong
		return
	}
	//clear[15] = ^random + 0x5a
	//clear[17] = 0x1a + random
	//util.GetCRC(clear[:18], 18)
	//clear[18] = ^(clear[18] + random - 15)
	//clear[19] = ^clear[19] - random
	random := clear[17] - 0x1a
	if clear[15] != ^random+0x5a {
		// todo logger 有人在非法请求
		clearStruct.CheckResult = ErrorTokenWrong
		return
	}
	clear[18] = ^clear[18] - random + 15
	clear[19] = ^(clear[19] + random)
	if util.CheckCRC(clear) == false {
		// todo logger
		clearStruct.CheckResult = ErrorTokenWrong
		return
	}

	//clear[11] = byte(expireTime) + random + 27
	//clear[5] = ^byte(expireTime >> 8)
	//clear[1] = byte(expireTime>>16) - random
	//clear[7] = byte(expireTime>>24) + random + 99
	clearStruct.ExpireTime = (uint32(clear[11] - 27 - random)) +
		(uint32(^clear[5]) << 8) +
		(uint32(clear[1]+random) << 16) +
		(uint32(clear[7]-random-99) << 24)
	if uint32(time.Now().Unix()) > clearStruct.ExpireTime { // 到期时间已经过了
		clearStruct.CheckResult = ErrorTokenRunTime
		return
	}

	//clear[10] = byte(creatTime) - 69
	//clear[14] = byte(creatTime>>8) - random - 69
	//clear[8] = byte(creatTime>>16) + random - 88
	//clear[3] = ^byte(creatTime>>24) - random
	//clear[6] = byte(creatTime>>32) + random + 66
	//clear[13] = ^byte(creatTime>>40) - random - 28
	//clear[4] = byte(creatTime >> 48) - random + 23
	clearStruct.CreatTime = int64(clear[10]+69) +
		(int64(clear[14]+69+random) << 8) +
		(int64(clear[8]+88-random) << 16) +
		(int64(^(clear[3] + random)) << 24) +
		(int64(clear[6]-66-random) << 32) +
		(int64(^(clear[13] + 28 + random)) << 40) +
		(int64(clear[4]-23+random) << 48)

	if time.Now().UnixMicro() < clearStruct.CreatTime { // 创建时间居然大于当前时间
		// todo logger
		clearStruct.CheckResult = ErrorTokenWrong
		return
	}

	//clear[9] = ^byte(userID)
	//clear[12] = byte(userID>>8) - random - 33
	//clear[2] = byte(userID>>16) + random + 166
	//clear[0] = ^byte(userID>>24) - random
	clearStruct.UserID = uint32(^clear[9]) +
		(uint32(clear[12]+33+random) << 8) +
		(uint32(clear[2]-166-random) << 16) +
		(uint32(^(clear[0] + random)) << 24)

	clearStruct.UserType = clear[16] + random

	if clearStruct.UserType != 1 && clearStruct.UserType != 2 {
		// todo logger
		clearStruct.CheckResult = ErrorTokenWrong
	}
	//if clearStruct.UserType == 2 {
	//	if clearStruct.CreatTime < cache.GetAdminTokenCreatTime(clearStruct.UserID) {
	//		clearStruct.CheckResult = errorTokenExit
	//		return
	//	}
	//} else if  clearStruct.UserType == 1 {
	//	if clearStruct.CreatTime < cache.GetWechatCustomerTokenCreatTime(clearStruct.UserID) {
	//		clearStruct.CheckResult = errorTokenExit
	//		return
	//	}
	//}
	return
}

func CheckTokenExit(clearStruct *TokenResult) {
	if clearStruct.UserType == 2 {
		if clearStruct.CreatTime < cache.GetAdminTokenCreatTime(clearStruct.UserID) {
			clearStruct.CheckResult = ErrorTokenExit
			return
		}
	} else if clearStruct.UserType == 1 {
		if clearStruct.CreatTime < cache.GetWechatCustomerTokenCreatTime(clearStruct.UserID) {
			clearStruct.CheckResult = ErrorTokenExit
			return
		}
	}
}

func JeffWTPublic() gin.HandlerFunc {
	return func(c *gin.Context) {
		clearStruct := DecryptToken(c.Request.Header.Get("token"))
		if clearStruct.CheckResult == Success {
			CheckTokenExit(&clearStruct)
		}
		if clearStruct.CheckResult != Success { //必须要这个判断，因为CheckTokenExit(&clearStruct)会改变
			c.JSON(http.StatusOK, gin.H{
				"status":  clearStruct.CheckResult,
				"message": CodeMsg[uint16(clearStruct.CheckResult)],
			})
			c.Abort()
			return
		}
		c.Set("userID", clearStruct.UserID)
		c.Next()
	}
}

func JeffWTWechat() gin.HandlerFunc {
	return func(c *gin.Context) {
		clearStruct := DecryptToken(c.Request.Header.Get("token"))
		if clearStruct.UserType != 1 {
			c.JSON(http.StatusOK, gin.H{
				"status":  ErrorUserRole,
				"message": CodeMsg[ErrorUserRole],
			})
			c.Abort()
			return
		}
		if clearStruct.CheckResult == Success {
			CheckTokenExit(&clearStruct)
		}
		if clearStruct.CheckResult != Success {
			c.JSON(http.StatusOK, gin.H{
				"status":  clearStruct.CheckResult,
				"message": CodeMsg[uint16(clearStruct.CheckResult)],
			})
			c.Abort()
			return
		}
		c.Set("userID", clearStruct.UserID)
		c.Next()
	}
}

func JeffWTAdmin() gin.HandlerFunc {
	return func(c *gin.Context) {
		clearStruct := DecryptToken(c.Request.Header.Get("token"))
		if clearStruct.UserType != 2 {
			c.JSON(http.StatusOK, gin.H{
				"status":  ErrorUserRole,
				"message": CodeMsg[ErrorUserRole],
			})
			c.Abort()
			return
		}
		if clearStruct.CheckResult == Success {
			CheckTokenExit(&clearStruct)
		}
		if clearStruct.CheckResult != Success {
			c.JSON(http.StatusOK, gin.H{
				"status":  clearStruct.CheckResult,
				"message": CodeMsg[uint16(clearStruct.CheckResult)],
			})
			c.Abort()
			return
		}
		c.Set("userID", clearStruct.UserID)
		c.Next()
	}
}

//package api
//
//import (
//	"dc_golang_server_1/cache"
//	"dc_golang_server_1/util"
//	"github.com/gin-gonic/gin"
//	"net/http"
//	"time"
//)
//
//const jeffWTKey = "jeffWT@666%b;'~+"
//
////const jeffWTTable = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
//
//const tokenLen = 60
//
//type TokenResult struct {
//	CreatTime int64 //token创建时间
//	UserID string //
//	Admin bool
//	Duration uint32
//}
//
//// CreateToken 生成token,duration=0表示不限时长
//func CreateToken(t TokenResult) string {
//	expireTime := uint32(t.CreatTime/1000000) + t.Duration
//
//	var clear = make([]byte, 44) //8字节创建时间 4字节结束时间 30字节userName 2字节CRC
//
//	//r := rand.New(rand.NewSource(time.Now().UnixNano()))
//	if len(t.UserID) > 30 {
//		return ""
//	}
//	random := byte(t.CreatTime)
//	var str [30]byte
//	for i := 0; i < len(t.UserID); i++ {
//		str[i] = t.UserID[i] + (uint8(i) << 2) + random
//	}
//
//	clear[0] = str[8]
//	clear[1] = str[25]
//	clear[2] = byte(expireTime) + random + 27
//	clear[3] = str[27]
//	clear[4] = str[15]
//	clear[5] = str[19]
//	clear[6] = str[11]
//	clear[7] = str[23]
//	clear[8] = str[6]
//	clear[9] = str[28]
//	clear[10] = byte(t.CreatTime>>32) + random + 66
//	clear[11] = str[16]
//	clear[12] = ^random + 0x5a
//	clear[13] = str[17]
//	clear[14] = str[3]
//	clear[15] = str[21]
//	clear[16] = str[1]
//	clear[17] = byte(expireTime>>16) - random
//	clear[18] = str[0]
//	clear[19] = str[4]
//	clear[20] = ^byte(expireTime >> 8)
//	clear[21] = str[5]
//	clear[22] = byte(len(t.UserID)) + random
//	clear[23] = str[7]
//	clear[24] = str[9]
//	clear[25] = str[10]
//	clear[26] = byte(expireTime>>24) + random + 99
//	clear[27] = str[12]
//	clear[28] = str[13]
//	clear[29] = str[14]
//	clear[30] = byte(t.CreatTime >> 48)
//	if t.Admin {
//		clear[30] |= 0x80
//	}
//	clear[31] = str[18]
//	clear[32] = str[20]
//	clear[33] = str[22]
//	clear[34] = str[2]
//	clear[35] = byte(t.CreatTime>>16) + random - 88
//	clear[36] = ^byte(t.CreatTime>>40) - random - 28
//	clear[37] = str[24]
//	clear[38] = str[26]
//	clear[39] = str[29]
//	clear[40] = ^byte(t.CreatTime>>24) - random
//	clear[41] = byte(t.CreatTime>>8) - random - 69
//	util.GetCRC(clear[:42], 42)
//	clear[42] = ^(clear[42] + random)
//	clear[43] = ^clear[43] - random
//
//	return util.EncryptBytesToBase64URL(clear, jeffWTKey)
//}
//
//func CheckToken(token string)TokenResult{
//	return TokenResult{}
//}
//
//func JeffWT() gin.HandlerFunc {
//	return func(c *gin.Context) {
//		//tokenHeader := c.Request.Header.Get("Authorization")
//		//if tokenHeader == "" {
//		//	c.JSON(http.StatusOK, gin.H{
//		//		"status":  1004,
//		//		"message": "token不存在",
//		//	})
//		//	c.Abort()
//		//	return
//		//}
//		//
//		//checkToken := strings.Split(tokenHeader, " ")
//		//if len(checkToken) == 0 {
//		//	c.JSON(http.StatusOK, gin.H{
//		//		"status":  1007,
//		//		"message": "TOKEN格式错误",
//		//	})
//		//	c.Abort()
//		//	return
//		//}
//		//
//		//if len(checkToken) != 2 || checkToken[0] != "Bearer" {
//		//	c.JSON(http.StatusOK, gin.H{
//		//		"status":  1007,
//		//		"message": "TOKEN格式错误",
//		//	})
//		//	c.Abort()
//		//	return
//		//}
//		//
//		//if len(checkToken[1]) != tokenLen {
//		//	// todoN logger 有人在非法请求
//		//	c.JSON(http.StatusOK, gin.H{
//		//		"status":  1004,
//		//		"message": "token长度错误",
//		//	})
//		//	c.Abort()
//		//	return
//		//}
//		tokenHeader := c.Request.Header.Get("token")
//		if len(tokenHeader) != tokenLen {
//			// todo logger 有人在非法请求
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1004,
//				"message": "token长度错误",
//			})
//			c.Abort()
//			return
//		}
//
//		clear, err := util.DecryptBase64URLToBytes(tokenHeader, jeffWTKey)
//		if err != nil {
//			// todo logger 有人在非法请求
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1006,
//				"message": "TOKEN不正确",
//			})
//			c.Abort()
//			return //errors.New("TOKEN格式错误")//err
//		}
//		if len(clear) != 44 {
//			// todo logger
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1006,
//				"message": "TOKEN不正确",
//			})
//			c.Abort()
//			return //errors.New("TOKEN格式错误")//err//return info,false //errors.New("TOKEN格式错误")//("明文长度错误")
//		}
//
//		random := clear[12] - 0x5a
//		clear[42] = ^clear[42] - random
//		clear[43] = ^(clear[43] + random)
//		if util.CheckCRC(clear) == false {
//			// todo logger
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1006,
//				"message": "TOKEN不正确",
//			})
//			c.Abort()
//			return ////errors.New("TOKEN格式错误")//("明文校验错误")
//		}
//
//		userNameLen := clear[22] - random
//		if userNameLen < 1 || userNameLen > 30 {
//			// todo logger
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1006,
//				"message": "TOKEN不正确",
//			})
//			c.Abort()
//			return ////errors.New("TOKEN格式错误")//("登录名长度错误")
//		}
//
//		var admin bool
//		if clear[30]&0x80 == 0x80 {
//			admin = true
//			clear[30] &= 0x7f
//		}
//
//		expireTime := (uint32(clear[2] - 27 - random)) +
//			(uint32(^clear[20]) << 8) +
//			(uint32(clear[17]+random) << 16) +
//			(uint32(clear[26]+random) << 24)
//		if expireTime > uint32(time.Now().Unix()) {
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1005,
//				"message": "登录已过期",
//			})
//			c.Abort()
//			return
//		}
//		creatTime := int64(random) +
//			(int64(clear[41]+69+random) << 8) +
//			(int64(clear[35]+88-random) << 16) +
//			(int64(^(clear[40] + random)) << 24) +
//			(int64(clear[10]-66-random) << 32) +
//			(int64(^(clear[36] + 28 + random)) << 40) +
//			(int64(clear[30]-8-random) << 48)
//
//		if expireTime < uint32(creatTime/1000000) {
//			// todo logger
//			c.JSON(http.StatusOK, gin.H{
//				"code":  1006,
//				"message": "TOKEN不正确",
//			})
//			c.Abort()
//			return //
//		}
//
//		var str [30]byte
//
//		str[8] = clear[0]
//		str[25] = clear[1]
//		str[27] = clear[3]
//		str[15] = clear[4]
//		str[19] = clear[5]
//		str[11] = clear[6]
//		str[23] = clear[7]
//		str[6] = clear[8]
//		str[28] = clear[9]
//		str[16] = clear[11]
//		str[17] = clear[13]
//		str[3] = clear[14]
//		str[21] = clear[15]
//		str[1] = clear[16]
//		str[0] = clear[18]
//		str[4] = clear[19]
//		str[5] = clear[21]
//		str[7] = clear[23]
//		str[9] = clear[24]
//		str[10] = clear[25]
//		str[12] = clear[27]
//		str[13] = clear[28]
//		str[14] = clear[29]
//		str[18] = clear[31]
//		str[20] = clear[32]
//		str[22] = clear[33]
//		str[2] = clear[34]
//		str[24] = clear[37]
//		str[26] = clear[38]
//		str[29] = clear[39]
//
//		for i := uint8(0); i < userNameLen; i++ {
//			str[i] = str[i] - random - (i << 2)
//		}
//		userName := string(str[:userNameLen])
//
//		if admin {
//			// todo 查cache 已退出登录
//		} else {
//			// todo 查cache 已退出登录
//			cacheCreatTime := cache.GetWechatCustomerTokenCreatTime(userName)
//			if cacheCreatTime > creatTime { //如果缓存里面存的CreatTime比token里面的CreatTime要晚，则token已退出登录
//				// todo logger warn
//				c.JSON(http.StatusOK, gin.H{
//					"code":  1006,
//					"message": "用户已退出登录",
//				})
//				c.Abort()
//				return //
//			}
//		}
//
//		c.Set("userID", userName)
//		c.Next()
//	}
//}
