package router

import (
	"dc_golang_server_1/api"
	"dc_golang_server_1/api/api_admin"
	"dc_golang_server_1/api/api_public"
	"dc_golang_server_1/api/api_we"
	_ "dc_golang_server_1/docs"
	"dc_golang_server_1/logger"
	config2 "dc_golang_server_1/util/config"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/swaggo/gin-swagger"
	"github.com/swaggo/gin-swagger/swaggerFiles"
	"time"
)

func InitRouter() { //*gin.Engine{
	gin.SetMode(config2.HttpServerMode)
	r := gin.New()
	r.Use(logger.HttpGinLog(), gin.Recovery())
	r.Use(api.Cors()) //全局跨域 局部跨域的话在路由组里面去添加
	//r.Use(CheckToken()) //jwt 还是在对应需要token的路由组添加

	if config2.HttpServerSwagger {
		r.GET("/swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))
	}

	base := r.Group("base/")
	base.Use(api.JeffWTPublic())
	{
		base.GET("province", api_public.GetProvinceList)
		base.GET("city/:id", api_public.GetCityList)
		base.GET("county/:id", api_public.GetCountyList)
		base.GET("locarea", api_public.GetAreaByLocation)
		base.GET("countyarea/:id", api_public.GetAreaByCounty)
	}

	r.POST("we/login/:code", api_we.WeLogin)
	wechat := r.Group("we/")
	wechat.Use(api.JeffWTWechat())
	{ // 需要token的
		wechat.POST("carduser", api_we.CreateCardUser)               //创建学生
		wechat.DELETE("carduser/:id", api_we.DeleteCardUser)         //删除学生
		wechat.GET("trend/:id", api_we.GetCardUsersTrend)            //获取学生最新动态
		wechat.GET("bindcarduser/:id", api_we.GetBindCardUser)       //获取学生副家长列表(仅主家长有权限)
		wechat.GET("sharecode/id", api_we.CardUserShareCode)         //获取邀请码(仅主家长有权限)
		wechat.POST("bindcarduser/:share_code", api_we.BindCardUser) //副家长绑定学生
		wechat.DELETE("bindcarduser/:id", api_we.DeleteBindCardUser) //副家长解绑学生
		//wechat.PUT("cardusershareon/:id", api_we.PutCardUserShareOn)   //打开邀请码开关(仅主家长有权限)
		//wechat.PUT("cardusershareoff/:id", api_we.PutCardUserShareOff) //关闭邀请码开关(仅主家长有权限)
		wechat.DELETE("slaveweuser", api_we.DeleteSlaveWeUser) //删除副家长(仅主家长有权限)
		//wechat.DELETE("allslaveweuser", api_we.DeleteAllSlaveWeUser)         //删除副家长(仅主家长有权限)
		wechat.PUT("carduser/:id", api_we.PutCardUser)               //修改学生信息
		wechat.PUT("cardloss/:id", api_we.CardLoss)                  //卡挂失/解挂
		wechat.PUT("cardbind/:id", api_we.CardBind)                  //绑定卡
		wechat.DELETE("cardbind/:id", api_we.DeleteCardBind)         //解绑卡
		wechat.GET("phonecardstatus/:id", api_we.GetPhoneCardStatus) //获取学生公话卡状态
		wechat.GET("callrecord/:id", api_we.GetCallRecord)           // 查询通话记录
		wechat.POST("familynum/:id", api_we.AddFamilyNum)            //添加亲情号
		wechat.PUT("familynum/:id", api_we.PutFamilyNum)             //修改亲情号
		wechat.DELETE("familynum/:id", api_we.DeleteFamilyNum)       //删除亲情号
		//缴费
		//查询缴费记录

		//获取学生定位卡状态
		//查询通行记录
		//缴费
		//查询缴费记录
	}

	// r.POST("ad/login", api_we.AdminLogin)
	admin := r.Group("ad/")
	// admin.Use(api.JeffWTAdmin())
	{
		admin.POST("area", api_admin.CreateArea) // 新建一个学校
		// admin.GET("cache", api_cache.GetCacheMap) // 获取缓存内容
	}

	fmt.Println(time.Now(), "server is running:swagger", config2.HttpServerSwagger)

	err := r.Run(config2.HttpServerPort)
	if err != nil {
		panic(err)
	}
}
