package cache

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/data_db_cache/model"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"time"
)

//id integer NOT NULL DEFAULT nextval('b_area_id_seq'::regclass),
//county_id integer,
//name character varying(32) COLLATE pg_catalog."default",
//longitude integer,
//latitude integer,
//status smallint,
//creator_id integer,
//updater_id integer,
//create_at timestamp without time zone,
//update_at timestamp without time zone,
//CONSTRAINT b_area_pkey PRIMARY KEY (id),
//CONSTRAINT b_area_uniq1 UNIQUE (name)

func initAreaMapAndCountyAreaMap() {
	fmt.Println(time.Now(), "cache: initAreaMap begin")
	var areaID uint32
	var temp model.CacheBAreaStruct
	rows, err := dbdao.PgDb.Query("SELECT id,county_id,name,longitude,latitude,status FROM b_area") //
	defer rows.Close()
	if err != nil {
		log.Panicln("SelectAreaToMap rows", err)
	}

	for rows.Next() {
		err = rows.Scan(&areaID, &temp.CountyID, &temp.Name, &temp.Longitude, &temp.Latitude, &temp.Status)
		if err != nil {
			log.Panicln("SelectAreaToMap rows.Scan", err)
		}
		fmt.Println(areaID, temp)
		AreaMap.Store(areaID, temp)
		if temp.CountyID != 0 {
			var areaIDList []uint32
			if v, ok := CountyAreaMap.Load(temp.CountyID); ok {
				areaIDList, _ = v.([]uint32)
			}
			areaIDList = append(areaIDList, areaID)
			CountyAreaMap.Store(temp.CountyID, areaIDList)
			fmt.Println(temp.CountyID, areaIDList)
		}
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		log.Panicln(time.Now(), "initAreaMap rows.Err()", err)
	}

	fmt.Println(time.Now(), "cache: initAreaMap OK")
}

func GetAreaMapName(areaID uint32, test bool) string {
	if v, ok := AreaMap.Load(areaID); ok {
		if data, ok := v.(model.CacheBAreaStruct); ok {
			if data.Status == 1 || (data.Status == 2 && test) {
				return data.Name
			}
		} else {
			AreaMap.Delete(areaID)
			logger.Log.Error("AreaMap 类型断言错误",
				zap.String("Src", "GetAreaMapName"),
				zap.Uint32("areaID", areaID))
		}
	} else {
		logger.Log.Error("AreaMap 没数据",
			zap.String("Src", "GetAreaMapName"),
			zap.Uint32("areaID", areaID))
	}

	// todo 查库
	return ""
}

func GetAreaMapInfo(areaID uint32, info *model.CacheBAreaStruct) bool {
	if v, ok := AreaMap.Load(areaID); ok {
		if *info, ok = v.(model.CacheBAreaStruct); ok {
			return true
		} else {
			AreaMap.Delete(areaID)
			logger.Log.Error("AreaMap 类型断言错误",
				zap.String("Src", "GetAreaMapInfo"),
				zap.Uint32("areaID", areaID))

			return false
		}
	} else {
		logger.Log.Error("AreaMap 没数据",
			zap.String("Src", "GetAreaMapInfo"),
			zap.Uint32("areaID", areaID))
	}

	// todo 查库
	return false
}
