package cache

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"time"
)

func initAreaServiceMap() {
	fmt.Println(time.Now(), "cache: initDeviceMap begin")
	var areaID int32
	var temp AreaServiceStruct
	rows, err := dbdao.PgDb.Query("SELECT id,name,status FROM b_area") //todo
	defer rows.Close()
	if err != nil {
		log.Panicln("SelectAreaToMap rows", err)
	}
	for rows.Next() {
		err = rows.Scan(&areaID, &temp.Name, &temp.Status)
		if err != nil {
			log.Panicln("SelectAreaToMap rows.Scan", err)
		}
		phone20DeviceMap.Store(areaID, temp)
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		logger.Log.Error("rows.Err()",
			zap.Error(err),
			zap.String("src", "initAreaMap"))
		fmt.Println(time.Now(), "initAreaMap rows.Err()", err)
	} else {
		fmt.Println(time.Now(), "cache: initAreaMap end")
	}
}

func GetAreaServiceInfo(areaServiceID uint32) (areaServiceInfo AreaServiceStruct) {
	if info, ok := areaServiceMap.Load(areaServiceID); ok { //map有
		if areaServiceInfo, ok = info.(AreaServiceStruct); ok { //类型断言正确
			if areaServiceInfo.Status == 0 {
				areaServiceMap.Delete(areaServiceID)
				logger.Log.Error("areaServiceMap areaServiceInfo.Status == 0",
					zap.Uint32("areaServiceID", areaServiceID),
					zap.String("src", "GetAreaServiceInfo"))
			} else {
				return areaServiceInfo
			}
		} else { //类型断言失败
			areaServiceMap.Delete(areaServiceID)
			logger.Log.Error("areaServiceMap 类型断言错误",
				zap.Uint32("areaServiceID", areaServiceID),
				zap.String("src", "GetAreaServiceInfo"))
		}
	}

	var err error
	for i := 0; i < 10; i++ {
		err = dbdao.PgDb.QueryRow("SELECT name,status FROM b_area WHERE id=$1",
			areaServiceID).Scan(&areaServiceInfo.Name, &areaServiceInfo.Status) //todo
		if err != nil {
			fmt.Println("GetAreaInfo  SELECT:", i, err)
			time.Sleep(3 * time.Millisecond)
			continue
		}
		areaServiceMap.Store(areaServiceID, areaServiceInfo)
		logger.Log.Error("数据库有而缓存没有 areaServiceMap",
			zap.Uint32("areaServiceID", areaServiceID),
			zap.Error(err),
			zap.String("src", "GetAreaServiceInfo"))
		return
	}
	logger.Log.Error("GetAreaServiceInfo SELECT operator_id,status FROM b_area WHERE id=$1", //todo
		zap.Uint32("areaServiceID", areaServiceID),
		zap.Error(err),
		zap.String("src", "GetAreaServiceInfo"))
	return
}
