package cache

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/data_db_cache/model"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"sync"
	"time"
)

// 初始化行政区域缓存
func initBDistrictMaps() {
	fmt.Println(time.Now(), "cache: initBDistrictMaps begin")

	go func() bool {
		ProvinceMap = []model.CacheBDistrictStruct{} //先清空原来的缓存
		var temp model.CacheBDistrictStruct
		rows, err := dbdao.PgDb.Query("SELECT id,name,initial FROM b_district WHERE level=1 ORDER BY initial,sort")
		defer rows.Close()
		if err != nil {
			log.Panicln("ProvinceMap SELECT b_district rows", err)
		}
		for rows.Next() {
			err = rows.Scan(&temp.Id, &temp.Name, &temp.Initial)
			if err != nil {
				log.Panicln("ProvinceMap SELECT b_district rows.Scan", err)
			}
			ProvinceMap = append(ProvinceMap, temp)
		}
		// 处理完毕后，需要判断一次遍历过程中是否有错误产生
		if err = rows.Err(); err != nil {
			log.Panicln("cache: ProvinceMap rows.Err()", err)
		}
		return true
	}()
	selectToDistrictMap(2, &CityMap)
	selectToDistrictMap(3, &CountyMap)

	fmt.Println(time.Now(), "cache: initBDistrictMaps OK")
}

func selectToDistrictMap(level uint8, tempMap *sync.Map) bool {
	// 清空
	tempMap.Range(func(k, _ interface{}) bool {
		tempMap.Delete(k)
		return true
	})

	var parentID uint32
	var temp model.CacheBDistrictStruct
	rows, err := dbdao.PgDb.Query("SELECT id,name,parent_id,initial FROM b_district WHERE level=$1 ORDER BY initial,sort", level)
	defer rows.Close()
	if err != nil {
		log.Panicln("SELECT b_district rows", err, level)
	}
	for rows.Next() {
		err = rows.Scan(&temp.Id, &temp.Name, &parentID, &temp.Initial)
		if err != nil {
			log.Panicln("SELECT b_district rows.Scan", err, level)
		}
		var bufStruct []model.CacheBDistrictStruct
		if v, ok := tempMap.Load(parentID); ok { //map已经有了
			if bufStruct, ok = v.([]model.CacheBDistrictStruct); ok == false { //类型断言错误
				tempMap.Delete(parentID)
				logger.Log.Error("[]BDistrictStruct 类型断言错误",
					zap.Uint8("level", level),
					zap.String("src", "selectToDistrictMap"))
			}
		}
		bufStruct = append(bufStruct, temp)
		tempMap.Store(parentID, bufStruct)
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		log.Panicln("cache: selectToDistrictMap rows.Err()", err, level)
	}
	return true
}
