package cache

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"time"
)

func initPhone20DeviceMap() {
	fmt.Println(time.Now(), "cache: initDeviceMap begin")
	var devID uint32
	var temp Phone20DeviceStruct
	rows, err := dbdao.PgDb.Query("SELECT id,area_service_id,allow_call_time,status FROM d_device")
	defer rows.Close()
	if err != nil {
		log.Panicln("SelectDeviceToMap rows", err)
	}
	for rows.Next() {
		err = rows.Scan(&devID, &temp.AreaServiceID, &temp.AllowCallTime, &temp.Status)
		if err != nil {
			log.Panicln("SelectDeviceToMap rows.Scan", err)
		}
		phone20DeviceMap.Store(devID, temp)
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		logger.Log.Error("rows.Err()",
			zap.Error(err),
			zap.String("src", "initDeviceMap"))
		fmt.Println(time.Now(), "initDeviceMap rows.Err()", err)
	} else {
		fmt.Println(time.Now(), "cache: initDeviceMap OK")
	}
}

func GetPhone20Info(devID uint32) Phone20DeviceStruct {
	if info, ok := phone20DeviceMap.Load(devID); ok { //map有
		if v, ok2 := info.(Phone20DeviceStruct); ok2 { //类型断言正确
			return v //,true
		} else { //类型断言失败
			phone20DeviceMap.Delete(devID)
			logger.Log.Error("DeviceMap 类型断言错误",
				zap.Uint32("devID", devID),
				zap.String("src", "getDeviceInfo"))
		}
	}
	var temp Phone20DeviceStruct
	var err error
	for i := 0; i < 100; i++ {
		err = dbdao.PgDb.QueryRow("SELECT area_service_id,allow_call_time,status FROM d_phone20 WHERE id=$1",
			devID).Scan(&temp.AreaServiceID, &temp.AllowCallTime, &temp.Status)
		if err != nil {
			fmt.Println("GetPhone20Info  SELECT:", i, err)
			time.Sleep(3 * time.Millisecond)
			continue
		}
		phone20DeviceMap.Store(devID, temp)
		logger.Log.Error("数据库有而缓存没有 GetPhone20Info",
			zap.Uint32("devID", devID),
			zap.Error(err),
			zap.String("src", "GetPhone20Info"))
		return temp //,true
	}
	logger.Log.Error("getDeviceInfo SELECT SELECT area_service_id,allow_call_time,status FROM d_phone20 WHERE id=$1",
		zap.Uint32("devID", devID),
		zap.Error(err),
		zap.String("src", "GetPhone20Info"))
	return temp //,false
}
