package cache

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/data_db_cache/model"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"time"
)

func initWechatCustomerMap() {
	fmt.Println(time.Now(), "cache: initWechatCustomerMap begin")
	var weUserID uint32
	var unionID string
	var temp model.CacheUWeCustomerStruct
	// 1. 查u_we_customer表获得基础数据
	rows, err := dbdao.PgDb.Query("SELECT id,unionid,phone,nickname,avatarurl,test_role FROM u_we_customer") //
	// defer rows.Close()
	if err != nil {
		rows.Close()
		log.Panicln("SelectWechatCustomerMap rows", err)
	}

	for rows.Next() {
		err = rows.Scan(&weUserID, &unionID, &temp.WePhone, &temp.Nickname, &temp.AvatarURL, &temp.TestRole)
		if err != nil {
			rows.Close()
			log.Panicln("SelectWechatCustomerMap rows.Scan", err)
		}
		if len(unionID) > 0 {
			WechatCustomerUnionIDToIDMAP.Store(unionID, weUserID)
			WechatCustomerMap.Store(weUserID, temp)
		}
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		rows.Close()
		log.Panicln(time.Now(), "initWechatCustomerMap rows.Err()", err)
	}
	rows.Close()

	// 2. 查家长学生对应关系表u_we_card_user表,获得绑定的学生数据
	rows, err = dbdao.PgDb.Query("SELECT we_user_id,card_user_id,master FROM u_we_card_user order by create_at") //
	defer rows.Close()
	if err != nil {
		log.Panicln("Select u_we_card_user  rows", err)
	}

	var cUserID uint32
	var master bool
	for rows.Next() {
		err = rows.Scan(&weUserID, &cUserID, &master)
		if err != nil {
			log.Panicln("Select u_we_card_user rows.Scan", err)
		}
		if master {
			if v, ok := WechatCustomerMap.Load(weUserID); ok {
				if value, ok := v.(model.CacheUWeCustomerStruct); ok {
					value.MasterCardUserID = append(value.MasterCardUserID, cUserID)
					WechatCustomerMap.Store(weUserID, value)
				} else {
					logger.Log.Error("u_we_card_user 表中的 we_user_id,在wechatCustomerMap类型断言错误",
						zap.Uint32("weUserID", weUserID),
						zap.Uint32("cUserID", cUserID),
						zap.Bool("master", master),
						zap.String("src", "initWechatCustomerMap"))
				}
			} else {
				logger.Log.Error("u_we_card_user 表中的 we_user_id,在u_we_customer表中没有",
					zap.Uint32("weUserID", weUserID),
					zap.Uint32("cUserID", cUserID),
					zap.Bool("master", master),
					zap.String("src", "initWechatCustomerMap"))
			}
		} else {
			if v, ok := WechatCustomerMap.Load(weUserID); ok {
				if value, ok := v.(model.CacheUWeCustomerStruct); ok {
					value.SlaveCardUserID = append(value.SlaveCardUserID, cUserID)
					WechatCustomerMap.Store(weUserID, value)
				} else {
					logger.Log.Error("u_we_card_user 表中的 we_user_id,在wechatCustomerMap类型断言错误",
						zap.Uint32("weUserID", weUserID),
						zap.Uint32("cUserID", cUserID),
						zap.Bool("master", master),
						zap.String("src", "initWechatCustomerMap"))
				}
			} else {
				logger.Log.Error("u_we_card_user 表中的 we_user_id,在u_we_customer表中没有",
					zap.Uint32("weUserID", weUserID),
					zap.Uint32("cUserID", cUserID),
					zap.Bool("master", master),
					zap.String("src", "initWechatCustomerMap"))
			}
		}
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		log.Panicln(time.Now(), "Select u_we_card_user  rows.Err()", err)
	}

	fmt.Println(time.Now(), "cache: initWechatCustomerMap OK")
}

func GetWechatCustomerTokenCreatTime(weUserID uint32) int64 {
	var data model.CacheUWeCustomerStruct
	if v, ok := WechatCustomerMap.Load(weUserID); ok {
		if data, ok = v.(model.CacheUWeCustomerStruct); ok {
			return data.TokenCreatTime //正常情况
		} else {
			logger.Log.Error("WechatCustomerMap 类型断言错误",
				zap.String("Src", "GetWechatCustomerTokenCreatTime"),
				zap.Uint32("userID", weUserID))
		}
	} else {
		logger.Log.Error("WechatCustomerMap 没数据",
			zap.String("Src", "GetWechatCustomerTokenCreatTime"),
			zap.Uint32("userID", weUserID))
	}

	return 0x7fffffffffffffff //没有这个用户
}

func GetWechatCustomerTestRole(userID uint32) (bool, bool) {
	var data model.CacheUWeCustomerStruct
	if v, ok := WechatCustomerMap.Load(userID); ok {
		if data, ok = v.(model.CacheUWeCustomerStruct); ok {
			return data.TestRole, true //正常情况
		} else {
			logger.Log.Error("WechatCustomerMap 类型断言错误",
				zap.String("Src", "GetWechatCustomerTestRole"),
				zap.Uint32("userID", userID))
		}
	} else {
		logger.Log.Error("WechatCustomerMap 没数据",
			zap.String("Src", "GetWechatCustomerTestRole"),
			zap.Uint32("userID", userID))
	}

	// 再查一遍库
	unionID := dbdao.SelectUWeCustomerByUserIDToCacheModel(userID, &data)
	if len(unionID) == 0 {
		return false, false
	}

	WechatCustomerUnionIDToIDMAP.Store(unionID, userID)
	WechatCustomerMap.Store(userID, data)
	return data.TestRole, true
}

func GetWechatCustomerUserIDByUnionID(unionID string) uint32 {
	if v, ok := WechatCustomerUnionIDToIDMAP.Load(unionID); ok {
		if userID, ok := v.(uint32); ok {
			return userID
		}
	}
	return 0
}

func GetWechatCustomerInfoAndCanUpdateTokenTime(weUserID uint32, newTime int64) (model.CacheUWeCustomerStruct, bool) {
	var data model.CacheUWeCustomerStruct
	if v, ok := WechatCustomerMap.Load(weUserID); ok {
		if data, ok = v.(model.CacheUWeCustomerStruct); ok {
			if newTime != 0 {
				data.TokenCreatTime = newTime
				WechatCustomerMap.Store(weUserID, data)
			}
			return data, true //正常的话就该这里
		} else {
			logger.Log.Error("WechatCustomerMap 类型断言错误",
				zap.String("Src", "GetWechatCustomerInfo"),
				zap.Uint32("weUserID", weUserID))
			// todoN 考虑查一遍库
		}
	} else {
		logger.Log.Error("WechatCustomerMap 没数据",
			zap.String("Src", "GetWechatCustomerInfo"),
			zap.Uint32("weUserID", weUserID))
	}
	// 再查一遍库
	unionID := dbdao.SelectUWeCustomerByUserIDToCacheModel(weUserID, &data)
	if len(unionID) == 0 {
		return data, false
	}

	data.TokenCreatTime = newTime
	WechatCustomerUnionIDToIDMAP.Store(unionID, weUserID)
	WechatCustomerMap.Store(weUserID, data)
	return data, true
}

func InsertWechatCustomerMap(unionID string, userInfo *model.CacheUWeCustomerStruct) uint32 {
	userID, res := dbdao.InsertUWeCustomerUnionID(unionID)
	if res == dbdao.InsertSysErr {
		return 0
	}
	if res == dbdao.InsertDuplicate {
		if userID = dbdao.SelectUWeCustomerByUnionIDToCacheModel(unionID, userInfo); userID == 0 {
			return 0
		}
	}
	WechatCustomerUnionIDToIDMAP.Store(unionID, userID)
	WechatCustomerMap.Store(userID, *userInfo)

	return userID
}

// InsertNewStudentIDToWechatCustomerMap 缓存定制方法:微信用户增加/绑定卡用户
func InsertNewStudentIDToWechatCustomerMap(weUserID uint32, cardUserID uint32, master bool) bool {
	var data model.CacheUWeCustomerStruct
	if v, ok := WechatCustomerMap.Load(weUserID); ok {
		if data, ok = v.(model.CacheUWeCustomerStruct); ok {
			if master {
				data.MasterCardUserID = append(data.MasterCardUserID, cardUserID)
			} else {
				data.SlaveCardUserID = append(data.SlaveCardUserID, cardUserID)
			}
			WechatCustomerMap.Store(weUserID, data)
			return true //正常的话就该这里
		} else {
			logger.Log.Error("WechatCustomerMap 类型断言错误",
				zap.String("Src", "InsertNewStudentIDToWechatCustomerMap"),
				zap.Uint32("weUserID", weUserID))
			// todoN 考虑查一遍库
		}
	} else {
		logger.Log.Error("WechatCustomerMap 没数据",
			zap.String("Src", "InsertNewStudentIDToWechatCustomerMap"),
			zap.Uint32("weUserID", weUserID))
	}
	// 再查一遍库,库里面已经添加了学生数据了，就不用再添加了
	unionID := dbdao.SelectUWeCustomerByUserIDToCacheModel(weUserID, &data)
	if len(unionID) == 0 {
		return false
	}
	WechatCustomerUnionIDToIDMAP.Store(unionID, weUserID)
	WechatCustomerMap.Store(weUserID, data)
	return true
}
