package dbdao

import (
	"dc_golang_server_1/logger"
	"go.uber.org/zap"
)

//id integer NOT NULL DEFAULT nextval('u_card_user_id_seq'::regclass),
//role smallint 角色:0学生,1教师,2职工
//name character varying(16) COLLATE pg_catalog."default",
//sex smallint,
//area_id integer,
//grade smallint,
//class smallint,
//student_num character varying(32) COLLATE pg_catalog."default",
//create_at timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
//first_topup_at timestamp without time zone,

type InsertTableUserCardUser struct {
	Role   uint8  `swaggerignore:"true"`      // 后台添加选填,学生名字,角色:0学生,1教师,2职工
	Name   string `json:"name" example:"汤圆"`  // 必填,学生名字,maxLen=16
	AreaID uint32 `json:"areaID" example:"1"` // 必填,校区ID
	Grade  int8   `json:"grade"`              // 必填，"1~9对应一年级~九年级，10~12对应高一~高三"
	Sex    uint8  `json:"sex,omitempty"`      // 选填,1女2男
	Class  int8   `json:"class,omitempty"`    // 选填,班级,number
	StuNum string `json:"stuNum,omitempty"`   // 选填,学号,字符串,maxLen=32
}

// InsertUserCardUserAndUserWeCardUser 事务处理两张表
func InsertUserCardUserAndUserWeCardUser(insertData InsertTableUserCardUser, weUserID uint32) (cardUserID uint32) {
	tx, err := PgDb.Begin()
	if err != nil {
		logger.Log.Error("InsertUCallRecordAndUpdateUSimInfo PgDb.Begin():",
			zap.Error(err))
		return
	}
	defer clearTransaction(tx)

	err = tx.QueryRow("INSERT INTO u_card_user(role,name,sex,area_id,grade,class,student_num)VALUES($1,$2,$3,$4,$5,$6,$7)RETURNING id",
		insertData.Role,
		insertData.Name,
		insertData.Sex,
		insertData.AreaID,
		insertData.Grade,
		insertData.Class,
		insertData.StuNum,
	).Scan(&cardUserID)
	if err != nil {
		logger.Log.Error("InsertUserCardUserAndUserWeCardUser INSERT u_card_user:",
			zap.Error(err))
		return
	}

	_, err = tx.Exec("INSERT INTO u_we_card_user(we_user_id,card_user_id,master)VALUES($1,$2,true)", weUserID, cardUserID)
	if err != nil {
		logger.Log.Error("InsertUserCardUserAndUserWeCardUser INSERT u_we_card_user:",
			zap.Error(err))
		return 0
	}

	if err = tx.Commit(); err != nil {
		logger.Log.Error("InsertUserCardUserAndUserWeCardUser Commit:",
			zap.Error(err))
		return 0
	}

	return
}
