package devproduct

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/logger"
	"dc_golang_server_1/util"
	"encoding/hex"
	"go.uber.org/zap"
	"net"
	"strconv"
	"time"
)

//type IDCLongTCPHex interface {
//	receiveResponse(conn *net.TCPConn, data []byte, cnt int) bool
//}

type DCLongTCPHex struct {
	TCPLongServer
	password         string
	iv               string
	devType          uint8
	protocolAnalysis func(data *HexData) //HexData
}

//type Connection struct {
//	conn *net.TCPConn
//}

type HexData struct {
	TimeUtc uint32
	//DevType uint8
	DevID    uint32
	CtrlCode uint8
	Length   uint8
	Data     []byte
}

func /*(d *DCLongTCPHex)*/ insertDevHexCommandRecord(data *HexData, ciphertext string, direction uint8) {
	go dbdao.InsertDevCommandRecord(dbdao.TableDevHexCommandRecord{
		DevID:      data.DevID,
		DevUtc:     data.TimeUtc,
		CtrlCode:   strconv.FormatUint(uint64(data.CtrlCode), 16),
		DataLen:    data.Length,
		DataHex:    hex.EncodeToString(data.Data[:data.Length]),
		Ciphertext: ciphertext,
		Direction:  direction,
	})
}

func (d *DCLongTCPHex) receiveResponse(conn *net.TCPConn, data []byte, nowConnDevID uint32) uint32 {
	c, err := util.DecryptBase64ToBytes(data, d.password, d.iv)
	if err != nil {
		if nowConnDevID != 0 {
			dbdao.InsertDevCommandErr(nowConnDevID, string(data), "Decrypt err")
		}
		return 0
	}
	if util.CheckCRC(c) == false {
		dbdao.InsertDevCommandErr(nowConnDevID, string(data), "CheckCRC err")
		return 0
	}
	cLen := len(c)
	if cLen < 12 {
		dbdao.InsertDevCommandErr(nowConnDevID, string(data), "cLen<12")
		return 0
	} else if int(c[9])+12 != cLen {
		dbdao.InsertDevCommandErr(nowConnDevID, string(data), "cLen!=c[9]+12")
		return 0
	}

	var rData HexData
	rData.TimeUtc = uint32(c[3])<<24 | uint32(c[2])<<16 | uint32(c[1])<<8 | uint32(c[0]) // binary.Read(bytes.NewBuffer(cleartext[:4]), binary.BigEndian, &rData.timeUtc) 肯定效率低下，还是自己拼

	// 时间有效性判断
	nowTimeUtc := uint32(time.Now().Unix())
	if nowTimeUtc > rData.TimeUtc+300 || rData.TimeUtc > nowTimeUtc+300 {
		dbdao.InsertDevCommandErr(nowConnDevID, string(data), "utc err:"+strconv.FormatUint(uint64(rData.TimeUtc), 10))
		return 0
	}

	// 设备类型判断
	if c[4] != d.devType {
		dbdao.InsertDevCommandErr(nowConnDevID, string(data), "devType error")
		return 0
	}

	rData.DevID = uint32(c[4])<<24 | uint32(c[5])<<16 | uint32(c[6])<<8 | uint32(c[7])
	rData.CtrlCode = c[8]
	rData.Length = c[9]
	rData.Data = c[10 : cLen-2]

	if rData.CtrlCode != 0x41 && rData.CtrlCode != 0x3f { //心跳不记 开机注册不记
		insertDevHexCommandRecord(&rData, string(data), 0) //0接收  1回复  2主动下发  异步去搞
	}
	/*sBuf := */ d.protocolAnalysis(&rData)
	if rData.CtrlCode > 0x80 { // 用来判断返回不为空 被动响应都大于0x80
		sendStr := d.packageSendData(&rData)
		sendStr[0] = '['
		sendStr[len(sendStr)-1] = ']'

		//if sendFlag {time.Sleep(20*time.Millisecond)} //这句话是为了多帧发送间隔20MS,可以不加试试

		if _, err = conn.Write(sendStr); err != nil {
			//logger.Log.Debug("conn.Write(sendStr) error", // 刚收到发来的，现在居然发不回去了
			//	zap.String("Src", "TCP-SERVER-receive"),
			//	zap.Uint32("DevID", rData.DevID),
			//	zap.String("RData", string(data)),
			//	zap.String("SendStr", string(sendStr)),
			//	zap.String("Conn", conn.RemoteAddr().String()))
			//if nowConnDevID == 0 { //刚才没有ID的在这里记
			//	model.InsertDevCommandErr(rData.DevID,string(data),"A error")
			//}
			return 0
		}

		if rData.CtrlCode != 0xc1 && rData.CtrlCode != 0xbf { //心跳不记 开机注册不记
			insertDevHexCommandRecord(&rData, string(sendStr), 1) //0接收  1回复  2主动下发 异步去搞
		}
	}

	return rData.DevID //解密成功且CRC校验成功，设备类型一致，解析长度正确，则为合法设备
}

func (d *DCLongTCPHex) TCPSend(data *HexData) bool {
	//d.ConnectMapRWMutex.RLock()
	//c,ok := d.ConnectMap[data.DevID]// conn,ok := d.ConnectMap.Load(data.DevID)
	//d.ConnectMapRWMutex.RUnlock()
	if conn, ok := d.ConnectMap.Load(data.DevID); ok {
		if c, ok := conn.(*net.TCPConn); ok { // 类型断言
			sendStr := d.packageSendData(data)
			sendStr[0] = '['
			sendStr[len(sendStr)-1] = ']'
			if _, err := c.Write(sendStr); err != nil {
				//d.ConnectMapRWMutex.Lock()
				//delete(d.ConnectMap,data.DevID)// d.ConnectMap.Delete(data.DevID)
				//d.ConnectMapRWMutex.Unlock()
				/*errC*/
				_ = c.Close()
				d.ConnectMap.Delete(data.DevID)
				/*if errC != nil {
					model.DevConnectRecord(data.DevID,false,'W')
				} else {
					model.DevConnectRecord(data.DevID,false,'w')
				}*/
				//logger.Log.Debug("conn.Write(sendStr) error", //
				//	zap.String("Src", "TCP-SERVER-TCPSend"),
				//	zap.Uint32("DevID", data.DevID),
				//	zap.Error(err),
				//	zap.String("SendStr", string(sendStr)),
				//	zap.String("Conn", c.RemoteAddr().String()),
				//	zap.String("State", "offline"))
				return false
			}
			insertDevHexCommandRecord(data, string(sendStr), 2) //0接收  1回复  2主动下发
			return true
		} else {
			// ntodo logger map里面存的内容居然类型不对
			d.ConnectMap.Delete(data.DevID)
			logger.Log.Error("conn.(*net.TCPConn) != ok", //
				zap.String("Src", "TCP-SERVER-TCPSend"),
				zap.Uint32("DevID", data.DevID),
				zap.String("Conn", c.RemoteAddr().String()),
				zap.String("State", "offline"))
		}
	}

	return false
}

func (d *DCLongTCPHex) InitNewDCLongTCPHex(password string, iv string, devType uint8, f func(data *HexData)) {
	d.password = password
	d.iv = iv
	d.devType = devType
	d.protocolAnalysis = f
	d.TCPReceive = d.receiveResponse
	// d.ConnectMap = make(map[uint32]*net.TCPConn)
}

// 组装并加密数据
func (d *DCLongTCPHex) packageSendData(sData *HexData) []byte {
	hexBuf := make([]byte, sData.Length+12)
	sData.TimeUtc = uint32(time.Now().Unix())
	hexBuf[0] = byte(sData.TimeUtc)
	hexBuf[1] = byte(sData.TimeUtc >> 8)
	hexBuf[2] = byte(sData.TimeUtc >> 16)
	hexBuf[3] = byte(sData.TimeUtc >> 24)
	hexBuf[4] = d.devType
	hexBuf[5] = byte(sData.DevID >> 16)
	hexBuf[6] = byte(sData.DevID >> 8)
	hexBuf[7] = byte(sData.DevID)
	hexBuf[8] = sData.CtrlCode
	hexBuf[9] = sData.Length
	//if sData.Length>0 {
	copy(hexBuf[10:], sData.Data[:sData.Length])
	//}
	util.GetCRC(hexBuf, int(sData.Length)+10)

	return util.EncryptBytesToBase64(hexBuf, d.password, d.iv)
}
