package dcphone20

import (
	"dc_golang_server_1/data_db_cache/cache"
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/devproduct"
	"dc_golang_server_1/logger"
	"fmt"
	"github.com/axgle/mahonia"
	"go.uber.org/zap"
	"strconv"
	"strings"
	"sync"
	"time"
)

//type DCPhone10StateStruct struct{
//	rTime time.Time
//	csq0 uint8
//	csq1 uint8
//	state uint8
//	m1state uint8
//	m2state uint8
//	errCode uint8
//}

type DevStateStruct struct {
	Csq0             byte      `json:"csq0"`
	Csq1             byte      `json:"csq1"`
	CallModuleStatus byte      `json:"call_module_status"`
	DevErrorCode     byte      `json:"dev_error_code"`
	EnableKeyboard   bool      `json:"enable_keyboard"`
	CallIn           bool      `json:"call_in"`
	EnableSos        bool      `json:"enable_sos"`
	EnableCall       bool      `json:"enable_call"`
	EnableMsg        bool      `json:"enable_msg"`
	UpdateTime       time.Time `json:"update_time"`
}

type DCPhone20 struct {
	devproduct.DCLongTCPHex
	MapDCPhone10State sync.Map
}

func (dev *DCPhone20) NewDCPhone20(password string, iv string, devType uint8) {
	dev.InitNewDCLongTCPHex(password, iv, devType, dev.response)
}

//TCP收到的协议解析
func (dev *DCPhone20) response(data *devproduct.HexData) {
	ctrlCode := data.CtrlCode
	data.CtrlCode = 0
	switch ctrlCode {
	// 参数类 para
	case 0x03 | 0x80: // "tcp_server_addr", // 硬件连接tcp服务器地址端口 //"\"water-tcp-short.dcrym.com\",8013"
		if data.Length > 10 { // len(string(data.Data))
			//stringsTemp := strings.SplitN(string(data.Data)[1:], "\",", 2) // [1:data.Length]
			//ip := "error"
			//var port uint16
			//if len(stringsTemp) == 2 {
			//	if p, err := strconv.ParseUint(stringsTemp[1], 10, 16); err == nil {
			//		port = uint16(p)
			//		ip = stringsTemp[0]
			//	}
			//}
			//jsonStruct := paraTcpServerAddr{
			//	data.TimeUtc,
			//	[]uint32{data.DevID},
			//	struct {
			//		IP   string `json:"ip"`
			//		Port uint16 `json:"port"`
			//	}{ip, port},
			//}
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x04 | 0x80: // "heart", // 心跳间隔次数 和 心跳间隔时间
		if data.Length == 3 {
			//jsonStruct := paraHeart{
			//	data.TimeUtc,
			//	[]uint32{data.DevID},
			//	struct {
			//		Time      uint16 `json:"time"`       // 推荐60
			//		VoidCount byte   `json:"void_count"` // 推荐0
			//	}{uint16(data.Data[0]) + (uint16(data.Data[1]) << 8), data.Data[2]},
			//}
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x05 | 0x80, 0x06 | 0x80, 0x07 | 0x80: //"beep_volume", // 蜂鸣器音量 0-100
		// 0x06|0x80, //"speech_volume", // 通话音量 0-10
		// 0x07|0x80:// "speech_speed", // 语速 0-9
		if data.Length == 1 {
			//jsonStruct := paraOneByte{
			//	data.TimeUtc,
			//	[]uint32{data.DevID},
			//	data.Data[0],
			//}
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x08 | 0x80:
		if data.Length == 1 {
			//jsonStruct := paraSelectSimModule{
			//	data.TimeUtc,
			//	[]uint32{data.DevID},
			//	struct {
			//		Net  byte `json:"net"`  // 0/1
			//		Call byte `json:"call"` // 0/1
			//	}{0, 0},
			//}
			//
			//if data.Data[0]&1 != 0 {
			//	jsonStruct.Value.Call = 1
			//}
			//if data.Data[0]&2 != 0 {
			//	jsonStruct.Value.Net = 1
			//}
			//
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x09 | 0x80: // "work_time", //工作时段
		//groupNum := data.Length / 6
		//workTimeTemp := make([]struct {
		//	Start          uint16 `json:"start"`
		//	Stop           uint16 `json:"stop"`
		//	EnableKeyboard bool   `json:"enable_keyboard"`
		//	CallIn         bool   `json:"call_in"`
		//	EnableSos      bool   `json:"enable_sos"`
		//	EnableCall     bool   `json:"enable_call"`
		//	EnableMsg      bool   `json:"enable_msg"`
		//	WeekDay        []bool `json:"week_day"`
		//}, groupNum)
		//var i uint8
		//for i = 0; i < groupNum; i++ {
		//	workTimeTemp[i].Start = uint16(data.Data[0+i*6]) + uint16(data.Data[1+i*6])<<8
		//	workTimeTemp[i].Stop = uint16(data.Data[2+i*6]) + uint16(data.Data[3+i*6])<<8
		//	if (data.Data[4+i*6] & 1) != 0 {
		//		workTimeTemp[i].EnableCall = true
		//	} // 允许刷卡
		//	if (data.Data[4+i*6] & (1 << 1)) != 0 {
		//		workTimeTemp[i].CallIn = true
		//	} // 允许呼入
		//	if (data.Data[4+i*6] & (1 << 2)) != 0 {
		//		workTimeTemp[i].EnableKeyboard = true
		//	} // 允许键盘拨号
		//	if (data.Data[4+i*6] & (1 << 3)) != 0 {
		//		workTimeTemp[i].EnableMsg = true
		//	} // 允许留言
		//	if (data.Data[4+i*6] & (1 << 4)) != 0 {
		//		workTimeTemp[i].EnableSos = true
		//	} // 允许紧急呼叫
		//	workTimeTemp[i].WeekDay = make([]bool, 7)
		//	for j := 0; j < 7; j++ {
		//		if (data.Data[5+i*6] & (1 << j)) != 0 {
		//			workTimeTemp[i].WeekDay[j] = true
		//		}
		//	}
		//	//fmt.Println(workTimeTemp[i].WeekDay)
		//}
		//
		//jsonStruct := paraWorkTime{
		//	DevTime: data.TimeUtc,
		//	DevID:   []uint32{data.DevID},
		//}
		//jsonStruct.Value = workTimeTemp
		//x, _ := json.Marshal(jsonStruct)
		//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
	case 0x0a | 0x80: // "key_password", // 最长10个字符(数字加*#)
		//jsonStruct := paraOneString{
		//	data.TimeUtc,
		//	[]uint32{data.DevID},
		//	string(data.Data),
		//}
		//x, _ := json.Marshal(jsonStruct)
		//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)

		// 设备控制类ctrl
	case 0x32 | 0x80: // 0x32|0x80:"ctrl/restart_device", // 重启设备 上下行均空
		//jsonStruct := paraNoValue{
		//	data.TimeUtc,
		//	[]uint32{data.DevID},
		//}
		//x, _ := json.Marshal(jsonStruct)
		//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
	case 0x70 | 0x80: // 0x70|0x80:"ctrl/read_firmware_version", // 读取固件版本
		if data.Length > 22 {
			//jsonStruct := devUpFirmwareInfo{
			//	data.TimeUtc,
			//	[]uint32{data.DevID},
			//	struct {
			//		MD5     []byte `json:"md5"`
			//		Length  uint32 `json:"length"`
			//		Version string `json:"version"`
			//	}{
			//		data.Data[0:16],
			//		uint32(data.Data[16]) + (uint32(data.Data[17]) << 8) + (uint32(data.Data[18]) << 16) + (uint32(data.Data[19]) << 24),
			//		string(data.Data[20:]),
			//	},
			//}
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x71 | 0x80: // "ctrl/update_firmware", // 启动远程更新
		if data.Length > 2 {
			//jsonStruct := devUpRespondUpdateFirmware{
			//	DevTime: data.TimeUtc,
			//	DevID:   []uint32{data.DevID},
			//}
			//
			//if data.Data[0] == 1 {
			//	jsonStruct.Value.RespondUpdateFirmware = true
			//	jsonStruct.Value.Version = string(data.Data[1 : data.Length-1])
			//} else if data.Data[0] == 2 && data.Length > 23 {
			//	jsonStruct.Value.MD5 = data.Data[1:17]
			//	jsonStruct.Value.Length = uint32(data.Data[17]) + (uint32(data.Data[18]) << 8) + (uint32(data.Data[19]) << 16) + (uint32(data.Data[20]) << 24)
			//	jsonStruct.Value.Version = string(data.Data[20:])
			//} else {
			//	logger.Log.Error("data.Data error",
			//		zap.String("Src", "TCP-SERVER-response"),
			//		zap.Uint32("DevID", data.DevID),
			//		zap.Uint8("CtrlCode", ctrlCode),
			//		zap.Uint8("data.Data[0]", data.Data[0]),
			//		zap.Uint8s("RecvData", data.Data))
			//}
			//
			//x, _ := json.Marshal(jsonStruct)
			//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x30 | 0x80:
		//jsonStruct := structOnlineDial{
		//	data.TimeUtc,
		//	[]uint32{data.DevID},
		//	struct {
		//		Result uint8  `json:"result"`
		//		Time   uint8  `json:"time"`
		//		Number string `json:"number"`
		//	}{
		//		data.Data[0],
		//		data.Data[1],
		//		string(data.Data[2:]),
		//	},
		//}
		//x, _ := json.Marshal(jsonStruct)
		//_, _ = jeffhttprequest.Post(config.HttpClientBaseUrl+httpParaNameMapHex2String[ctrlCode], x)
	case 0x3f: // 0x3f:"ctrl/dev_restart_login",// 开机注册
		if data.Length > 75 {
			// todoN 考虑第0字节增加模组选择
			insertDevHexResetRecord(data)
			data.CtrlCode = ctrlCode | 0x80
			data.Length = 0
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x41: // "ctrl/dev_up_status",// 心跳上报状态 更新DCPhone10State
		if data.Length > 2 { // 至少3个字节
			newState := DevStateStruct{
				UpdateTime: time.Now(), //.Format("2006-01-02 15:04:05.000000"),
				Csq0:       data.Data[0],
				Csq1:       data.Data[1],
			}
			for i := 0; i < 5; i++ {
				if (data.Data[2] & 1) != 0 {
					newState.EnableCall = true
				} // 允许刷卡
				if (data.Data[2] & (1 << 1)) != 0 {
					newState.CallIn = true
				} // 允许呼入
				if (data.Data[2] & (1 << 2)) != 0 {
					newState.EnableKeyboard = true
				} // 允许键盘拨号
				if (data.Data[2] & (1 << 3)) != 0 {
					newState.EnableMsg = true
				} // 允许留言
				if (data.Data[2] & (1 << 4)) != 0 {
					newState.EnableSos = true
				} // 允许紧急呼叫
			}
			data.CtrlCode = ctrlCode | 0x80
			data.Length = 0
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}

		//设备使用类 usedevice
	case 0x20: // "usedevice/dev_get_family_num",// 查询亲情号码 IC卡
		if data.Length == 5 {

		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x25: // 查询亲情号码 SIM ICCID
		if data.Length == 21 {
			data.CtrlCode = ctrlCode | 0x80
			enc := mahonia.NewEncoder("gbk")

			// 1.根据设备编号查status和AreaServiceID 3.根据ICCID查Status、MonthTalkMinutes、MonthPackageMinutes、family_num
			v := cache.GetPhone20Info(data.DevID)
			if v.AreaServiceID == 0 { // 后台没添加该设备,返回显示 话机设备未注册
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("话机设备未注册")))
				return
			} // 后台没添加该设备,返回显示 话机设备未注册
			if v.Status == false { //设备已停用,返回显示 设备暂停使用
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("设备暂停使用")))
				return
			} //设备已停用,返回显示 设备暂停使用
			if v.AllowCallTime == 0 {
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("设备无允许通话时长")))
				return
			}

			//2.根据AreaServiceID查status
			area := cache.GetAreaServiceInfo(v.AreaServiceID)
			if area.Status == 0 || area.ServiceID != 1 {
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("该区域暂未开通公话业务")))
				return
			}
			if area.Status == 1 {
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("该区域公话业务暂停使用")))
				return
			}

			s := cache.GetSimInfo(string(data.Data[1:]))
			if s.Status == 0 { // 这张卡不对，显示 未查询到亲情号码
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("未查询到亲情号码")))
				return
			} // 这张卡不对，显示 未查询到亲情号码
			if s.Status > 1 { //用户卡暂停使用
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("用户卡暂停使用")))
				return
			} //用户卡暂停使用
			if s.MonthTalkMinutes >= s.MonthPackageMinutes { //本月套餐无剩余通话分钟数
				data.Data[0] = 0xff
				data.Length = 21 + byte(copy(data.Data[21:], enc.ConvertString("本月套餐无剩余通话分钟数")))
				return
			} //本月套餐无剩余通话分钟数

			tempIccId := make([]byte, 20)
			copy(tempIccId, data.Data[1:])
			data.Data = make([]byte, 255)
			userCanUseTime := v.AllowCallTime
			if (s.MonthPackageMinutes - s.MonthTalkMinutes) < uint16(userCanUseTime) {
				userCanUseTime = uint8(s.MonthPackageMinutes - s.MonthTalkMinutes)
			}
			if len(s.FamilyNum) > 3 {
				logger.Log.Error("len(sim.FamilyNum)>3",
					zap.Uint32("AreaServiceID", v.AreaServiceID),
					zap.Uint32("DevID", data.DevID),
					zap.Uint8("CtrlCode", ctrlCode),
					zap.Uint8("DataLen", data.Length),
					zap.Uint8s("RecvData", data.Data),
					zap.String("FamilyNum", fmt.Sprintf("%s", s.FamilyNum)),
					zap.String("src", "tcp case 0x25"))
				data.Data[0] = 3
			} else {
				data.Data[0] = uint8(len(s.FamilyNum))
			}
			copy(data.Data[1:], tempIccId)
			var i uint8
			data.Length = 21
			for ; i < data.Data[0]; i++ {
				if i != 0 {
					data.Data[data.Length] = '-'
					data.Length++
				}
				data.Length += byte(copy(data.Data[data.Length:], s.FamilyNum[i].Phone))
				data.Data[data.Length] = '_'
				data.Length++
				data.Length += byte(copy(data.Data[data.Length:], strconv.Itoa(int(userCanUseTime))))
				data.Data[data.Length] = '_'
				data.Length++
				strNickname := enc.ConvertString(s.FamilyNum[i].Nickname)
				if len(strNickname) < 17 {
					data.Length += byte(copy(data.Data[data.Length:], strNickname))
				}
			}
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x23: // 0x23:"usedevice/dev_get_family_msg",// 查询留言
		if data.Length == 4 {
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x24: // 上报已读留言
		if data.Length == 8 {
		} else {
			data.CtrlCode = ctrlCode | 0x80
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x21: // 0x21:"usedevice/dev_get_call_time",// 获取自由拨号允许时长
		if data.Length > 4 {
		} else {
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x22: // 0x22:"usedevice/dev_post_call_log",// 上报通话记录
		if data.Length > 11 {

		} else {
			data.CtrlCode = ctrlCode | 0x80 //还是回硬件
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0x26: // 0x22:"usedevice/dev_post_call_log",// 上报通话记录 SIM ICCID
		if data.Length > 27 { //todo
			//devInfo := cache.GetDeviceInfo(data.DevID)
			//var areaInfo cache.CacheBAreaStruct
			//if devInfo.AreaID > 0 {
			//	areaInfo = cache.GetAreaInfo(devInfo.AreaID)
			//}
			//familyInfo := cache.GetSimFamilyNum(devInfo.AreaID,string(data.Data[1:21]))
			//calledNum := string(data.Data[27:])
			//var nickname string
			//for i:=0;i < len(familyInfo.Family);i++{
			//	if calledNum == familyInfo.Family[i].Phone {
			//		nickname = familyInfo.Family[i].Nickname
			//		break
			//	}
			//}
			//// 以事务方式存2张表
			//if result := dbdao.InsertUCallRecordAndUpdateUSimInfo(dbdao.TableUserCallRecord{
			//	//CustomerId:
			//	//CardUserId:
			//	//CardUserName:
			//	CardID: string(data.Data[1:21]),
			//	CardType:0,
			//	CalledNumber: calledNum,
			//	CalledNickname: nickname,
			//	CallStartAt: time.Unix(int64(data.Data[21]) + int64(data.Data[22])<<8 + int64(data.Data[23])<<16 + int64(data.Data[24])<<24,0).Format("2006-01-02 15:04:05"),
			//	TalkTime: int16(data.Data[25]) + int16(data.Data[26]<<8),
			//	AreaId: devInfo.AreaID,
			//	OperatorId: areaInfo.OperatorID,
			//	DeviceId: data.DevID,
			//	Status: data.Data[0],
			//});result > 1 {
			//	return
			//} else if result == 0 { //扣套餐分钟数
			//
			//}
			data.CtrlCode = ctrlCode | 0x80
		} else {
			data.CtrlCode = ctrlCode | 0x80 //还是回硬件
			logger.Log.Error("data.Length error",
				zap.String("Src", "TCP-SERVER-response"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("CtrlCode", ctrlCode),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data))
		}
	case 0xff: // 硬件被动上报异常
		logger.Log.Warn("dev up error 0xff",
			zap.String("Src", "TCP-SERVER-response"),
			zap.Uint32("DevID", data.DevID),
			zap.Uint8("CtrlCode", ctrlCode),
			zap.Uint8("DataLen", data.Length),
			zap.Uint8s("RecvData", data.Data))
	default:
		logger.Log.Error("ctrlCode error",
			zap.String("Src", "TCP-SERVER-response"),
			zap.Uint32("DevID", data.DevID),
			zap.Uint8("CtrlCode", ctrlCode),
			zap.Uint8("DataLen", data.Length),
			zap.Uint8s("RecvData", data.Data))
	}
}

func insertDevHexResetRecord(data *devproduct.HexData) {
	var resetReason byte
	if data.Data[0]&0x03 == 0x03 {
		resetReason = 0 // 正常上电
	} else if data.Data[0] == 0x40 {
		resetReason = 1 // 软件复位
	} else {
		resetReason = 2 // 异常复位
	}
	//imei0 := string(data.Data[1:16]) //从索引1到索引16(不包含)
	//iccid0 := string(data.Data[16:36])
	//imei1 := string(data.Data[36:51])
	//iccid1 := string(data.Data[51:71])
	var firmwareVersion string
	var longitude, latitude uint32
	strTemp := strings.Split(string(data.Data[71:]), "-") //data.Data[71:data.Length]
	if len(strTemp) == 3 {
		d, err := strconv.ParseFloat(strTemp[0], 64)
		if err != nil {
			logger.Log.Error("Longitude ParseFloat error",
				zap.String("Src", "InsertDevResetRecord"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data),
				zap.Uint32("DevUtc", data.TimeUtc))
		} else {
			longitude = uint32(d * 1000000)
		}

		d, err = strconv.ParseFloat(strTemp[1], 64)
		if err != nil {
			logger.Log.Error("Latitude ParseFloat error",
				zap.String("Src", "InsertDevResetRecord"),
				zap.Uint32("DevID", data.DevID),
				zap.Uint8("DataLen", data.Length),
				zap.Uint8s("RecvData", data.Data),
				zap.Uint32("DevUtc", data.TimeUtc))
		} else {
			latitude = uint32(d * 1000000)
		}
		firmwareVersion = strTemp[2] //
	}
	go dbdao.InsertDevResetRecord(dbdao.TableDevHexResetRecord{
		DevId:       data.DevID,
		DevUtc:      data.TimeUtc,
		ResetReason: resetReason,
		ResetCode:   strconv.FormatUint(uint64(data.Data[0]), 16),
		Location:    [2]uint32{longitude, latitude},
		Firmware:    firmwareVersion,
	})
}
