package devproduct

import (
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/logger"
	config2 "dc_golang_server_1/util/config"
	"go.uber.org/zap"
	"net"
	"strconv"
	"sync"
	"sync/atomic"
	"time"
)

type ITCPLongServer interface {
	TCPStart(port string)
}

type receiveCallbackFunc func(*net.TCPConn, []byte, uint32) uint32 //定义接收到数据的回调函数的函数类型

type TCPLongServer struct {
	TCPReceive      receiveCallbackFunc
	ConnSocketCount int32
	ConnectMap      sync.Map // 用这个可能性能不行，测试下，不行就用map加锁
	// rwConnSocketCount chan int // 20210815 将对ConnSocketCount的操作修改为原子操作
	// ConnectMapRWMutex sync.RWMutex
	// ConnectMap map[uint32]*net.TCPConn // Connection
}

func (s *TCPLongServer) TCPStart(port string) {
	go func() {
		// 获取一个TCP的Addr
		addr, err := net.ResolveTCPAddr("tcp4", "0.0.0.0:"+port)
		if err != nil {
			panic("error: net.ResolveTCPAddr " + port + err.Error())
		}

		// 监听服务器地址
		listener, err := net.ListenTCP("tcp4", addr)
		if err != nil {
			logger.Log.Panic("net.ListenTCP:"+port,
				zap.String("Src", "TCP-SERVER"),
				zap.Error(err))
		}

		// 启动server
		for {
			conn, err1 := listener.AcceptTCP() // 阻塞等待客户端建立连接请求
			if err1 != nil {
				logger.Log.Warn("listener.AcceptTCP()",
					zap.String("Src", "TCP-SERVER"),
					zap.Error(err1))
				continue
			}

			go s.connectionHandle(conn, s.TCPReceive)
		}
	}()
}

// 主动断开某链接，其实在新连接上来的时候，可以查下如果map里面有现在的设备号，就可以把旧的断掉
//func (s *TCPLongServer) DeleteConnection(key uint32){
//	if conn,ok := s.ConnectMap.Load(key);ok {
//		if c,ok := conn.(*net.TCPConn);ok{ // 类型断言
//			c.Close()
//		}
//	}
//	s.ConnectMap.Delete(key)
//	log.WithFields(log.Fields{
//		"Src": "TCP-SERVER-DeleteConnection",
//		"DevID": key,
//		"Conn":conn.RemoteAddr(),
//		"State": "offline",
//	}).Debugln("conn.Write(sendStr) error")
//}

func (s *TCPLongServer) connectionHandle(conn *net.TCPConn, callBack receiveCallbackFunc) {
	var myDevID uint32 // golang 默认0
	atomic.AddInt32(&s.ConnSocketCount, 1)
	defer func() {
		//s.rwConnSocketCount <- -1
		if s.ConnSocketCount > 0 {
			atomic.AddInt32(&s.ConnSocketCount, -1)
		} else {
			logger.Log.Error("s.ConnSocketCount error",
				zap.String("Src", "TCP-SERVER-connectionHandle"),
				zap.Uint32("DevID", myDevID),
				zap.String("Conn", conn.RemoteAddr().String()),
				zap.Int32("ConnSocketCount", s.ConnSocketCount))
		}
		if myDevID != 0 {
			if connMyID, ok := s.ConnectMap.Load(myDevID); ok { //查一下设备旧链接还在的话，关掉
				if co, ok := connMyID.(*net.TCPConn); ok { // 类型断言
					if co == conn {
						s.ConnectMap.Delete(myDevID)
						dbdao.DevConnectRecord(myDevID, 1) // state 0:新连接来了 1:设备正常离线 2:正常离线 3:新连接挤掉就连接 4:未知异常(连接池类型断言失败)
						//logger.Log.Debug("device offline",
						//	zap.String("Src", "TCP-SERVER-connectionHandle"),
						//	zap.Uint32("DevID", myDevID),
						//	zap.String("Conn", conn.RemoteAddr().String()),
						//	zap.String("State", "offline"))
					} else {
						// s.ConnectMap.Delete(myDevID) 这里人家存了新的，不能去删了噻
						dbdao.DevConnectRecord(myDevID, 3) // state 0:新连接来了 1:设备正常离线 2:正常离线 3:新连接挤掉就连接 4:未知异常(连接池类型断言失败)
						//logger.Log.Debug("have a new link,close old link",
						//	zap.String("Src", "TCP-SERVER-connectionHandle"),
						//	zap.Uint32("DevID", myDevID),
						//	zap.String("Conn", conn.RemoteAddr().String()),
						//	zap.String("State", "offline"))
					}
				} else {
					s.ConnectMap.Delete(myDevID)
					//logger.Log.Error("connMyID.(*net.TCPConn) != ok",
					//	zap.String("Src", "TCP-SERVER-connectionHandle"),
					//	zap.Uint32("DevID", myDevID),
					//	zap.String("Conn", conn.RemoteAddr().String()),
					//	zap.String("State", "offline")) // 类型断言失败
					dbdao.DevConnectRecord(myDevID, 4) // state 0:新连接来了 1:设备正常离线 2:正常离线 3:新连接挤掉就连接 4:未知异常(连接池类型断言失败)
				}
			} else {
				//logger.Log.Debug("map no have myDevID",
				//	zap.String("Src", "TCP-SERVER-connectionHandle"),
				//	zap.Uint32("DevID", myDevID),
				//	zap.String("Conn", conn.RemoteAddr().String()),
				//	zap.String("State", "offline"))
				dbdao.DevConnectRecord(myDevID, 2) // state 0:新连接来了 1:设备正常离线 2:正常离线 3:新连接挤掉就连接 4:未知异常(连接池类型断言失败)
			}

		}
		// _ = conn.SetReadDeadline(time.Time{}) // 设置回去，但是不知道需不需要这句话
		_ = conn.Close()
		// fmt.Println("-----------------------------------Out ConnSocketCount:", s.ConnSocketCount)
	}()

	if s.ConnSocketCount > config2.TcpLongMaxConnections { //限制最大连接数2万个配置文件 最大连接限制数添加到配置文件
		logger.Log.Error("ConnSocketCount overrun:"+strconv.Itoa(int(config2.TcpLongMaxConnections)),
			zap.String("Src", "TCP-SERVER-connectionHandle"),
			zap.Uint32("DevID", myDevID),
			zap.String("Conn", conn.RemoteAddr().String()),
			zap.Int32("ConnSocketCount", s.ConnSocketCount))
		return
	}

	_ = conn.SetKeepAlive(false)
	buf := make([]byte, 512)
	rBuf := make([]byte, 512)
	rLen := 0
	i := 0
	timeConn := time.Now().Unix() + config2.TcpLongNewConnRightfulTimeout //配置文件 120秒
	for {
		if myDevID == 0 {
			if time.Now().Unix() > timeConn { // 连上来2分钟都没整个正确的数据来，就给它断球
				logger.Log.Warn("NewConnRightfulTimeout:"+strconv.Itoa(int(config2.TcpLongNewConnRightfulTimeout)), // 超时没收到合法的指令
					zap.String("Src", "TCP-SERVER-connectionHandle"),
					// zap.Uint32("DevID", myDevID),
					zap.String("Conn", conn.RemoteAddr().String()),
					zap.Int32("ConnSocketCount", s.ConnSocketCount))
				return
			}
			if err := conn.SetReadDeadline(time.Now().Add(config2.TcpLongNewConnReadDeadline)); err != nil { //  配置文件 新连接1分钟超时未收到数据(配置文件用秒)
				logger.Log.Warn("conn.SetReadDeadline:new", //
					zap.String("Src", "TCP-SERVER-connectionHandle"),
					zap.Uint32("DevID", myDevID),
					zap.String("Conn", conn.RemoteAddr().String()),
					zap.Error(err),
					zap.Int32("ConnSocketCount", s.ConnSocketCount))
				return
			}
		} else {
			if err := conn.SetReadDeadline(time.Now().Add(config2.TcpLongOldConnReadDeadline)); err != nil { //  配置文件 老连接3分钟超时未收到数据(配置文件用秒)
				logger.Log.Warn("conn.SetReadDeadline:old", //
					zap.String("Src", "TCP-SERVER-connectionHandle"),
					zap.Uint32("DevID", myDevID),
					zap.String("Conn", conn.RemoteAddr().String()),
					zap.Error(err),
					zap.Int32("ConnSocketCount", s.ConnSocketCount))
				return
			}
		}
		cnt, err := conn.Read(buf)
		if err != nil {
			// fmt.Println("conn.Read:", err)
			// logger debug 其中存在超时，或新连接来关闭了旧链接
			logger.Log.Debug("read error",
				zap.String("Src", "TCP-SERVER-connectionHandle"),
				zap.Uint32("DevID", myDevID),
				zap.String("Conn", conn.RemoteAddr().String()),
				zap.Error(err))
			return
		}

		for i = 0; i < cnt; i++ {
			if rBuf[0] != '{' || rLen == 0 {
				if buf[i] == '{' {
					rBuf[0] = '{'
					rLen = 1
				} else {
					rLen = 0
				}
			} else if buf[i] == '}' {
				if rLen == 1 {
					if i+1 == cnt { //只有当后面没跟其他数据的时候，才回空心跳
						if _, err2 := conn.Write([]byte("[]")); err2 != nil {
							return
						}
					}
				} else {
					newID := callBack(conn, rBuf[1:rLen], myDevID)
					if newID == 0 {
						return
					} else if newID != myDevID {
						if myDevID == 0 {
							if connOld, ok := s.ConnectMap.Load(newID); ok { //查一下设备旧链接还在的话，关掉
								if co, ok1 := connOld.(*net.TCPConn); ok1 { // 类型断言
									s.ConnectMap.Store(newID, conn) //先存新的再关
									_ = co.Close()
								} /* else {
									s.ConnectMap.Delete(newID)
									fmt.Printf("newID ： s.ConnectMap.类型断言(%08X)错误\n",newID)
									// logger 类型断言不OK
								}*/
							}
							// logger 新链接来的合法设备
							dbdao.DevConnectRecord(newID, 0)
							//logger.Log.Debug("a new link is coming",
							//	zap.String("Src", "TCP-SERVER-connectionHandle"),
							//	zap.Uint32("DevID", newID),
							//	zap.String("Conn", conn.RemoteAddr().String()),
							//	zap.String("State", "online"))
						} else {
							s.ConnectMap.Delete(myDevID)
							logger.Log.Error("DevID changed",
								zap.String("Src", "TCP-SERVER-connectionHandle"),
								zap.Uint32("OldDevId", myDevID),
								zap.Uint32("NewDevId", newID),
								zap.String("Conn", conn.RemoteAddr().String()))
						}
						s.ConnectMap.Store(newID, conn)
						myDevID = newID
					} else { //这个地方应该查下map里面有没有，没有就是没对
						if connMyID, ok := s.ConnectMap.Load(myDevID); ok { //
							if co, ok2 := connMyID.(*net.TCPConn); ok2 { // 类型断言
								if co != conn {
									s.ConnectMap.Store(myDevID, conn)
									logger.Log.Error("co != conn",
										zap.String("Src", "TCP-SERVER-connectionHandle"),
										zap.Uint32("DevID", myDevID),
										zap.String("Conn", conn.RemoteAddr().String()),
										zap.String("State", "restore myDevId"))
								}
							} else {
								s.ConnectMap.Store(myDevID, conn)
								logger.Log.Error("connMyID.(*net.TCPConn) != ok",
									zap.String("Src", "TCP-SERVER-connectionHandle"),
									zap.Uint32("DevID", myDevID),
									zap.String("Conn", conn.RemoteAddr().String()),
									zap.String("State", "restore myDevId")) // 类型断言失败
							}
						} else {
							s.ConnectMap.Store(myDevID, conn)
							logger.Log.Error("map no myDevID",
								zap.String("Src", "TCP-SERVER-connectionHandle"),
								zap.Uint32("DevID", myDevID),
								zap.String("Conn", conn.RemoteAddr().String()),
								zap.String("State", "restore myDevId"))
						}
					}
				}
				rLen = 0
			} else if buf[i] == '{' {
				rBuf[0] = '{'
				rLen = 1
			} else {
				rBuf[rLen] = buf[i]
				if rLen < 512 {
					rLen++
				} else {
					logger.Log.Warn("rBufOverrun:512",
						zap.String("Src", "TCP-SERVER-connectionHandle"),
						zap.Uint8s("RBuf", rBuf),
						zap.Int("RLen", rLen),
						// zap.Uint32("DevID", myDevID),
						zap.String("Conn", conn.RemoteAddr().String()))
					rLen = 0
				}
			}
		}
	}
}
