package config

import (
	"go.uber.org/zap/zapcore"
	"gopkg.in/ini.v1"
	"time"
)

var (
	DbDSN string

	HttpServerMode    string
	HttpServerPort    string
	HttpServerSwagger bool

	TcpLongPort                   string
	TcpLongMaxConnections         int32
	TcpLongNewConnReadDeadline    time.Duration
	TcpLongNewConnRightfulTimeout int64
	TcpLongOldConnReadDeadline    time.Duration

	LogLevel       zapcore.Level
	LogPath        string
	LogFileMaxSize int // 单个日志文件的最大MB

	LogInfoMaxFileNum int
	LogInfoMaxFileDay int

	LogWarnMaxFileNum int
	LogWarnMaxFileDay int

	LogErrorMaxFileNum int
	LogErrorMaxFileDay int
)

func Init() {
	file, err := ini.Load("./config.ini")
	if err != nil {
		// logger.Log.Panic("配置文件读取错误")
		panic("配置文件读取错误")
	}

	DbDSN = file.Section("database").Key("dbDSN").MustString("user=higher password=hj^&bhjb789j$^* dbname=jeff_study host=pgm-2ze3yp0bss39cxvh1o.pg.rds.aliyuncs.com port=1921 sslmode=disable")

	HttpServerMode = file.Section("http_server").Key("ServerMode").In("release", []string{"release", "debug"})
	HttpServerPort = file.Section("http_server").Key("ServerPort").MustString(":50000")
	HttpServerSwagger = file.Section("http_server").Key("Swagger").MustBool(true)

	TcpLongPort = file.Section("tcp_long_server").Key("TcpPort").MustString("60000")
	TcpLongMaxConnections = int32(file.Section("tcp_long_server").Key("MaxConnections").MustUint(20000))
	TcpLongNewConnReadDeadline = time.Duration(file.Section("tcp_long_server").Key("NewConnReadDeadline").MustInt(60)) * time.Second
	TcpLongNewConnRightfulTimeout = file.Section("tcp_long_server").Key("NewConnRightfulTimeout").MustInt64(120)
	TcpLongOldConnReadDeadline = time.Duration(file.Section("tcp_long_server").Key("OldConnReadDeadline").MustInt(180)) * time.Second

	logLvlStr := file.Section("log").Key("Level").In("warn", []string{"trace", "debug", "info", "warn", "error"})
	switch logLvlStr {
	case "debug":
		LogLevel = zapcore.DebugLevel
	case "info":
		LogLevel = zapcore.InfoLevel
	case "warn":
		LogLevel = zapcore.WarnLevel
	case "error":
		LogLevel = zapcore.ErrorLevel
	}
	LogPath = file.Section("log").Key("Path").MustString(".log")
	LogFileMaxSize = file.Section("log").Key("FileMaxSize").MustInt(10)

	LogInfoMaxFileNum = file.Section("log").Key("InfoMaxFileNum").MustInt(10)
	LogInfoMaxFileDay = file.Section("log").Key("InfoWithMaxAge").MustInt(30)

	LogWarnMaxFileNum = file.Section("log").Key("WarnMaxFileNum").MustInt(10)
	LogWarnMaxFileDay = file.Section("log").Key("WarnWithMaxAge").MustInt(90)

	LogErrorMaxFileNum = file.Section("log").Key("ErrorMaxFileNum").MustInt(10)
	LogErrorMaxFileDay = file.Section("log").Key("ErrorWithMaxAge").MustInt(180)
}
